@@ -68,10 +68,10 @@ class Plugin(Plugin):
                 views.ApiElement.as_view()),
             url(rx + r'api/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>[^/]+)/(?P<field>\w+)/suggestions$',
                 views.Suggestions.as_view()),
-            url(rx + r'choices/(?P<app_label>\w+)/(?P<rptname>\w+)$',
+            url(rx + r'choices/(?P<app_label>\w+)/(?P<actor>\w+)$',
                 views.Choices.as_view()),
-            url(rx + r'choices/(?P<app_label>\w+)/(?P<rptname>\w+)/'
-                     '(?P<fldname>\w+)$',
+            url(rx + r'choices/(?P<app_label>\w+)/(?P<actor>\w+)/'
+                     '(?P<field>\w+)$',
                 views.Choices.as_view()),
             url(rx + r'apchoices/(?P<app_label>\w+)/(?P<actor>\w+)/'
                      '(?P<an>\w+)/(?P<field>\w+)$',
@@ -84,9 +84,6 @@ class Plugin(Plugin):
             url(rx+ r'choicelists/',
                 views.ChoiceListModel.as_view()),
 
-            # url(rx + "static/(?<Pstatic_path>.+)",
-            #     views.StaticRedirect.as_view())
-
         ]
         return urls
 
