@@ -5,17 +5,12 @@
 """Views for `lino_react.react`.
 """
 
-from __future__ import division
-from past.utils import old_div
-
-import logging
+# from __future__ import division
 
 from os import environ
 
 import ast
 
-logger = logging.getLogger(__name__)
-
 from django import http
 from django.db import models
 from django.conf import settings
@@ -249,178 +244,70 @@ class ApiList(View):
             return settings.SITE.kernel.run_action(ar)
 
         if fmt == constants.URL_FORMAT_JSON:
-            rows = [rh.store.row2list(ar, row)
+
+            window_type = request.GET.get(constants.URL_PARAM_WINDOW_TYPE, "g")
+
+            def serialize(ar, row):
+                """Use window_type to determin which serizlisation store metod and fields to use"""
+                if window_type == "g":
+                    return rh.store.row2list(ar, row)
+                else:
+                    return rh.store.row2dict(ar, row,
+                                             fields=rh.store.card_fields if window_type == "c" else rh.store.detail_fields,
+                                             card_title=ar.get_card_title(row)
+                                             )
+
+            rows = [serialize(ar, row)
                     for row in ar.sliced_data_iterator]
+
             total_count = ar.get_total_count()
-            for row in ar.create_phantom_rows():
-                if ar.limit is None or len(rows) + 1 < ar.limit or ar.limit == total_count + 1:
-                    d = rh.store.row2list(ar, row)
-                    rows.append(d)
-                total_count += 1
+            if window_type == "g":
+                for row in ar.create_phantom_rows():
+                    if ar.limit is None or len(rows) + 1 < ar.limit or ar.limit == total_count + 1:
+                        d = serialize(ar, row)
+                        rows.append(d)
+                    total_count += 1
             # assert len(rows) <= ar.limit
             kw = dict(count=total_count,
                       rows=rows,
                       success=True,
                       no_data_text=ar.no_data_text,
-                      # title=str(ar.get_title()),
                       title=ar.get_title())
+            if window_type != "g":
+                mc = ar.get_main_card()
+                if mc is not None:
+                    rows.insert(0, mc)
             if ar.actor.parameters:
                 kw.update(
                     param_values=ar.actor.params_layout.params_store.pv2dict(
                         ar, ar.param_values))
             return json_response(kw)
 
-        if fmt == constants.URL_FORMAT_HTML:
-            after_show = ar.get_status()
-
-            sp = request.GET.get(
-                constants.URL_PARAM_SHOW_PARAMS_PANEL, None)
-            if sp is not None:
-                # ~ after_show.update(show_params_panel=sp)
-                after_show.update(
-                    show_params_panel=constants.parse_boolean(sp))
-
-            # if isinstance(ar.bound_action.action, actions.ShowInsert):
-            #     elem = ar.create_instance()
-            #     rec = ar.elem2rec_insert(rh, elem)
-            #     after_show.update(data_record=rec)
-
-            kw = dict(on_ready=
-            ar.renderer.action_call(
-                ar.request,
-                ar.bound_action, after_show))
-            # ~ print '20110714 on_ready', params
-            kw.update(title=ar.get_title())
-            return http.HttpResponse(ar.renderer.html_page(request, **kw))
-
-        if fmt == 'csv':
-            # ~ response = HttpResponse(mimetype='text/csv')
-            charset = settings.SITE.csv_params.get('encoding', 'utf-8')
-            response = http.HttpResponse(
-                content_type='text/csv;charset="%s"' % charset)
-            if False:
-                response['Content-Disposition'] = \
-                    'attachment; filename="%s.csv"' % ar.actor
-            else:
-                # ~ response = HttpResponse(content_type='application/csv')
-                response['Content-Disposition'] = \
-                    'inline; filename="%s.csv"' % ar.actor
-
-            # ~ response['Content-Disposition'] = 'attachment; filename=%s.csv' % ar.get_base_filename()
-            w = ucsv.UnicodeWriter(response, **settings.SITE.csv_params)
-            w.writerow(ar.ah.store.column_names())
-            if True:  # 20130418 : also column headers, not only internal names
-                column_names = None
-                fields, headers, cellwidths = ar.get_field_info(column_names)
-                w.writerow(headers)
-
-            for row in ar.data_iterator:
-                w.writerow([str(v) for v in rh.store.row2list(ar, row)])
-            return response
-
-        if fmt == constants.URL_FORMAT_PRINTER:
-            if ar.get_total_count() > MAX_ROW_COUNT:
-                raise Exception(_("List contains more than %d rows") %
-                                MAX_ROW_COUNT)
-            response = http.HttpResponse(
-                content_type='text/html;charset="utf-8"')
-            doc = xghtml.Document(force_text(ar.get_title()))
-            doc.body.append(E.h1(doc.title))
-            t = doc.add_table()
-            # ~ settings.SITE.kernel.ar2html(ar,t,ar.data_iterator)
-            ar.dump2html(t, ar.data_iterator, header_links=False)
-            doc.write(response, encoding='utf-8')
-            return response
-
         return settings.SITE.kernel.run_action(ar)
 
 
 # Should we Refactor into lino.modlib.extjs.choicees_views.py and import?
-#
-# choices_for_field is copied line-for-line from lino.modlib.extjs.views.choices_for_field
-def choices_for_field(ar, holder, field):
-    """
-    Return the choices for the given field and the given HTTP request
-    whose `holder` is either a Model, an Actor or an Action.
-    """
-    if not holder.get_view_permission(ar.request.user.user_type):
-        raise Exception(
-            "{user} has no permission for {holder}".format(
-                user=ar.request.user, holder=holder))
-    # model = holder.get_chooser_model()
-    chooser = holder.get_chooser_for_field(field.name)
-    # logger.info('20140822 choices_for_field(%s.%s) --> %s',
-    #             holder, field.name, chooser)
-    if chooser:
-        qs = chooser.get_request_choices(ar, holder)
-        if not isiterable(qs):
-            raise Exception("%s.%s_choices() returned non-iterable %r" % (
-                holder.model, field.name, qs))
-        if chooser.simple_values:
-            def row2dict(obj, d):
-                d[constants.CHOICES_TEXT_FIELD] = str(obj)
-                d[constants.CHOICES_VALUE_FIELD] = obj
-                return d
-        elif chooser.instance_values:
-            # same code as for ForeignKey
-            def row2dict(obj, d):
-                d[constants.CHOICES_TEXT_FIELD] = holder.get_choices_text(
-                    obj, ar.request, field)
-                d[constants.CHOICES_VALUE_FIELD] = obj.pk
-                return d
-        else:  # values are (value, text) tuples
-            def row2dict(obj, d):
-                d[constants.CHOICES_TEXT_FIELD] = str(obj[1])
-                d[constants.CHOICES_VALUE_FIELD] = obj[0]
-                return d
-        return (qs, row2dict)
+# choices_for_field is copied line-for-line from
+# lino.modlib.extjs.views.choices_for_field
 
-    if field.choices:
-        qs = field.choices
+# 20200425 I saw no difference in the two versions, and I added two lines to the
+# choices_for_field in extjs.  IMO we should import it here. And before moving extjs
+# out of Lino, move choices_for_field to another module, e.g. to lino.core.fields.
 
-        def row2dict(obj, d):
-            if type(obj) is list or type(obj) is tuple:
-                d[constants.CHOICES_TEXT_FIELD] = str(obj[1])
-                d[constants.CHOICES_VALUE_FIELD] = obj[0]
-            else:
-                d[constants.CHOICES_TEXT_FIELD] = holder.get_choices_text(
-                    obj, ar.request, field)
-                d[constants.CHOICES_VALUE_FIELD] = str(obj)
-            return d
-
-        return (qs, row2dict)
-
-    if isinstance(field, fields.VirtualField):
-        field = field.return_type
-
-    if isinstance(field, fields.RemoteField):
-        field = field.field
-
-    if isinstance(field, models.ForeignKey):
-        m = field.remote_field.model
-        t = m.get_default_table()
-        qs = t.request(request=ar.request).data_iterator
-
-        # logger.info('20120710 choices_view(FK) %s --> %s', t, qs.query)
-
-        def row2dict(obj, d):
-            d[constants.CHOICES_TEXT_FIELD] = holder.get_choices_text(
-                obj, ar.request, field)
-            d[constants.CHOICES_VALUE_FIELD] = obj.pk
-            return d
-    else:
-        raise http.Http404("No choices for %s" % field)
-    return (qs, row2dict)
+from lino.modlib.extjs.views import choices_for_field
 
 
 # choices_response is copied line-for-line from lino.modlib.extjs.views.choices_response
-def choices_response(actor, request, qs, row2dict, emptyValue):
+def choices_response(actor, request, qs, row2dict, emptyValue, field=None):
     """
     :param actor: requesting Actor
     :param request: web request
     :param qs: list of django model QS,
     :param row2dict: function for converting data set into a dict for json
     :param emptyValue: The Text value to represent None in the choice-list
+    :param is_blank_value: The Text value to represent filtering rows that are blank in PV the choice-list
+    :param is_not_blank_value: The Text value to represent filtering rows that are not blank in PV the choice-list
+
     :return: json web responce
 
     Filters data-set acording to quickseach
@@ -432,6 +319,8 @@ def choices_response(actor, request, qs, row2dict, emptyValue):
     quick_search = request.GET.get(constants.URL_PARAM_FILTER, None)
     offset = request.GET.get(constants.URL_PARAM_START, None)
     limit = request.GET.get(constants.URL_PARAM_LIMIT, None)
+    wt = request.GET.get(constants.URL_PARAM_WINDOW_TYPE, None)
+
     if isinstance(qs, models.QuerySet):
         qs = qs.filter(qs.model.quick_search_filter(quick_search)) if quick_search else qs
         count = qs.count()
@@ -457,6 +346,18 @@ def choices_response(actor, request, qs, row2dict, emptyValue):
         rows = rows[int(offset):] if offset else rows
         rows = rows[:int(limit)] if limit else rows
 
+    if wt == constants.WINDOW_TYPE_PV and field and field.blank:
+        rows.insert(0, {
+            # constants.CHOICES_TEXT_FIELD: actor.get_blank_filter_text(),
+            constants.CHOICES_TEXT_FIELD: _("Blank"),
+            constants.CHOICES_VALUE_FIELD: constants.CHOICES_BLANK_FILTER_VALUE
+        })
+        rows.insert(1, {
+            # constants.CHOICES_TEXT_FIELD: actor.get_not_blank_filter_text(),
+            constants.CHOICES_TEXT_FIELD: _("Not Blank"),
+            constants.CHOICES_VALUE_FIELD: constants.CHOICES_NOT_BLANK_FILTER_VALUE
+        })
+
     # Add None choice
     if emptyValue is not None and not quick_search:
         empty = dict()
@@ -485,7 +386,7 @@ class ChoiceListModel(View):
 
 # Copied from lino.modlib.extjs.views.Choices line for line.
 class Choices(View):
-    def get(self, request, app_label=None, rptname=None, fldname=None, **kw):
+    def get(self, request, app_label=None, actor=None, field=None, **kw):
         """If `fldname` is specified, return a JSON object with two
         attributes `count` and `rows`, where `rows` is a list of
         `(display_text, value)` tuples.  Used by ComboBoxes or similar
@@ -495,9 +396,9 @@ class Choices(View):
         `record_selector` widget.
 
         """
-        rpt = requested_actor(app_label, rptname)
+        rpt = requested_actor(app_label, actor)
         emptyValue = None
-        if fldname is None:
+        if field is None:
             ar = rpt.request(request=request)
             # ~ rh = rpt.get_handle(self)
             # ~ ar = ViewReportRequest(request,rh,rpt.default_action)
@@ -517,15 +418,13 @@ class Choices(View):
             # NOTE: if you define a *parameter* with the same name as
             # some existing *data element* name, then the parameter
             # will override the data element here in choices view.
-            field = rpt.get_param_elem(fldname)
-            if field is None:
-                field = rpt.get_data_elem(fldname)
+            field = rpt.get_param_elem(field) or rpt.get_data_elem(field)
             if field.blank:
                 # logger.info("views.Choices: %r is blank",field)
                 emptyValue = ''
             qs, row2dict = choices_for_field(rpt.request(request=request), rpt, field)
 
-        return choices_response(rpt, request, qs, row2dict, emptyValue)
+        return choices_response(rpt, request, qs, row2dict, emptyValue, field=field)
 
 
 # Also coppied from extjs.views line for line
@@ -542,7 +441,7 @@ class ActionParamChoices(View):
             emptyValue = '<br/>'
         else:
             emptyValue = None
-        return choices_response(actor, request, qs, row2dict, emptyValue)
+        return choices_response(actor, request, qs, row2dict, emptyValue, field=field)
 
 
 class Restful(View):
@@ -856,8 +755,9 @@ class App(View):
                 # title=ar.get_title(),
                 # heading=ar.get_title(),
                 # main=main,
-                request = request,
-                user = user,  # Current user
+                front_end=ui,
+                request=request,
+                user=user,  # Current user
             )
             context.update(ar=ar)
 
@@ -884,6 +784,7 @@ class UserSettings(View):
         not_anon = u.is_authenticated if type(u.is_authenticated) == bool else u.is_authenticated()
 
         def getit():
+            # print(20200419, settings.SITE.build_media_url(*settings.SITE.plugins.react.renderer.lino_js_parts()))
             if not settings.SITE.build_js_cache_on_startup:
                 settings.SITE.plugins.react.renderer.build_js_cache(False)
             user_settings = dict(
@@ -908,13 +809,11 @@ class UserSettings(View):
                 user_settings["user_id"] = user_settings["su_id"] = su.id
                 user_settings["su_user_type"] = su.user_type
 
-
             if not_anon:
                 user_settings["authorities"] = u.get_authorities()
 
             return json_response(user_settings)
 
-
         return with_user_profile((su or u).user_type, getit)
 
 
